@@ -5,31 +5,27 @@ use Carp;
 
 use Tree::RB::Node qw[set_color color_of parent_of left_of right_of];
 use Tree::RB::Node::_Constants;
-use vars qw( $VERSION @EXPORT_OK );
-$VERSION = '0.500_005';
-$VERSION = eval $VERSION;
 
-require Exporter;
-*import    = \&Exporter::import;
-@EXPORT_OK = qw[LUEQUAL LUGTEQ LULTEQ LUGREAT LULESS LUNEXT LUPREV];
+our $VERSION = '0.1';
 
-use enum qw{
-    LUEQUAL
-    LUGTEQ 
-    LULTEQ 
-    LUGREAT
-    LULESS 
-    LUNEXT 
-    LUPREV 
+use Exporter 'import';
+our @EXPORT_OK = qw[LUEQUAL LUGTEQ LULTEQ LUGREAT LULESS LUNEXT LUPREV];
+
+use Data::Dumper;
+use constant {
+    LUEQUAL => 0,
+    LUGTEQ  => 1,
+    LULTEQ  => 2,
+    LUGREAT => 4,
+    LULESS  => 5,
+    LUNEXT  => 6,
+    LUPREV  => 7,
 };
-
-# object slots
-use enum qw{
-    ROOT
-    CMP 
-    SIZE
-    HASH_ITER
-    HASH_SEEK_ARG
+use constant {
+    ROOT  => 0,
+    CMP   => 1,
+    SIZE  => 2,
+    HASH_ITER => 3,
 };
 
 # Node and hash Iteration
@@ -39,23 +35,13 @@ sub _mk_iter {
     my $next_fn  = shift || 'successor';
     return sub {
         my $self = shift;
-        my $key  = shift;
         my $node;
         my $iter = sub {
             if($node) {
                 $node = $node->$next_fn;
             }
             else {
-                if(defined $key) {
-                    # seek to $key
-                    (undef, $node) = $self->lookup(
-                        $key, 
-                        $next_fn eq 'successor' ? LUGTEQ : LULTEQ
-                    );
-                } 
-                else {
-                    $node = $self->$start_fn;
-                }
+                $node = $self->$start_fn;
             }
             return $node;
         };
@@ -68,48 +54,10 @@ sub _mk_iter {
 *iter     = _mk_iter(qw/min successor/);
 *rev_iter = _mk_iter(qw/max predecessor/);
 
-sub hseek {
-    my $self = shift; 
-    my $arg  = shift;
-    defined $arg or croak("Can't seek to an undefined key");
-    my %args;
-    if(ref $arg eq 'HASH') {
-        %args = %$arg;
-    } 
-    else {
-        $args{-key} = $arg;
-    }
-    
-    if(@_ && exists $args{-key}) {
-        my $arg = shift;
-        if(ref $arg eq 'HASH') {
-            %args = (%$arg, %args);
-        } 
-    } 
-    if(! exists $args{-key}) {
-        defined $args{'-reverse'} or croak("Expected option '-reverse' is undefined");
-    }
-    $self->[HASH_SEEK_ARG] = \%args;
-    if($self->[HASH_ITER]) {
-        $self->_reset_hash_iter;
-    } 
-} 
-
-sub _reset_hash_iter {
-    my $self = shift; 
-    if($self->[HASH_SEEK_ARG]) {
-        my $iter = ($self->[HASH_SEEK_ARG]{'-reverse'} ? 'rev_iter' : 'iter');
-        $self->[HASH_ITER] = $self->$iter($self->[HASH_SEEK_ARG]{'-key'});
-    } 
-    else {
-        $self->[HASH_ITER] = $self->iter;
-    }
-} 
-
 sub FIRSTKEY {
     my $self = shift; 
-    $self->_reset_hash_iter;
 
+    $self->[HASH_ITER] = $self->iter;
     my $node = $self->[HASH_ITER]->next
       or return;
     return $node->[_KEY];
@@ -186,15 +134,12 @@ sub max {
 
 sub lookup {
     my $self = shift;
-    my $key  = shift;
-    defined $key
-      or croak("Can't use undefined value as key");
+    my $key  = shift or croak('Missing arg: $key');
     my $mode = shift || LUEQUAL;
     my $cmp = $self->[CMP];
 
     my $y;
-    my $x = $self->[ROOT]
-      or return;
+    my $x = $self->[ROOT];
     my $next_child;
     while($x) {
         $y = $x;
@@ -226,8 +171,7 @@ sub lookup {
             return wantarray ? ($y->[_VAL], $y) : $y->[_VAL];
         }
         else {
-            my $next = $y->successor
-              or return;
+            my $next = $y->successor;
             return wantarray ? ($next->[_VAL], $next) : $next->[_VAL];
         }
     }
@@ -236,8 +180,7 @@ sub lookup {
             return wantarray ? ($y->[_VAL], $y) : $y->[_VAL];
         }
         else {
-            my $next = $y->predecessor
-              or return;
+            my $next = $y->predecessor;
             return wantarray ? ($next->[_VAL], $next) : $next->[_VAL];
         }
     }
@@ -247,38 +190,6 @@ sub lookup {
 *FETCH = \&lookup;
 *get   = \&lookup;
 
-sub nth {
-    my ($self, $i) = @_;
-
-    $i =~ /^-?\d+$/
-      or croak('Integer index expected');
-    if ($i < 0) {
-        $i += $self->[SIZE];
-    }
-    if ($i < 0 || $i >= $self->[SIZE]) {
-        return;
-    }
-
-    my ($node, $next, $moves);
-    if ($i > $self->[SIZE] / 2) {
-        $node = $self->max;
-        $next = 'predecessor';
-        $moves = $self->[SIZE] - $i - 1;
-    }
-    else {
-        $node = $self->min;
-        $next = 'successor';
-        $moves = $i;
-    }
-
-    my $count = 0;
-    while ($count != $moves) {
-        $node = $node->$next;
-        ++$count;
-    }
-    return $node;
-}
-
 sub EXISTS {
     my $self = shift;
     my $key  = shift;
@@ -287,11 +198,11 @@ sub EXISTS {
 
 sub put {
     my $self = shift;
-    my $key_or_node = shift;
-    defined $key_or_node
-      or croak("Can't use undefined value as key or node");
+    my $key_or_node = shift or croak('key or node required');
     my $val = shift;
-
+    if(!$val && ref $key_or_node ne 'Tree::RB::Node') {
+        croak('value required');
+    }
     my $cmp = $self->[CMP];
     my $z = (ref $key_or_node eq 'Tree::RB::Node')
               ? $key_or_node
@@ -375,36 +286,16 @@ sub _fix_after_insertion {
 }
 
 sub delete {
-    my ($self, $key_or_node) = @_;
-    defined $key_or_node
-      or croak("Can't use undefined value as key or node");
-
+    my $self = shift;
+    my $key_or_node = shift or croak('key or node required');
     my $z = (ref $key_or_node eq 'Tree::RB::Node')
               ? $key_or_node
               : ($self->lookup($key_or_node))[1];
     return unless $z;
 
-    my $y;
-    if($z->[_LEFT] && $z->[_RIGHT]) {
-        # (Notes kindly provided by Christopher Gurnee)
-        # When deleting a node 'z' which has two children from a binary search tree, the
-        # typical algorithm is to delete the successor node 'y' instead (which is
-        # guaranteed to have at most one child), and then to overwrite the key/values of
-        # node 'z' (which is still in the tree) with the key/values (which we don't want
-        # to lose) from the now-deleted successor node 'y'.
-
-        # Since we need to return the deleted item, it's not good enough to overwrite the
-        # key/values of node 'z' with those of node 'y'. Instead we swap them so we can
-        # return the deleted values.
-
-        $y = $z->successor;
-        ($z->[_KEY], $y->[_KEY]) = ($y->[_KEY], $z->[_KEY]);
-        ($z->[_VAL], $y->[_VAL]) = ($y->[_VAL], $z->[_VAL]);
-    }
-    else {
-        $y = $z;
-    }
-
+    my $y = ($z->[_LEFT] && $z->[_RIGHT])
+              ? $z->successor
+              : $z;
     # splice out $y
     my $x = $y->[_LEFT] || $y->[_RIGHT];
     if(defined $x) {
@@ -552,6 +443,11 @@ __END__
 Tree::RB - Perl implementation of the Red/Black tree, a type of balanced binary search tree. 
 
 
+=head1 VERSION
+
+This document describes Tree::RB version 0.1
+
+
 =head1 SYNOPSIS
 
     use Tree::RB;
@@ -564,30 +460,27 @@ Tree::RB - Perl implementation of the Red/Black tree, a type of balanced binary
     $tree->put('Egypt'   => 'Cairo');
     $tree->put('Germany' => 'Berlin');
 
-    $tree->put('Alaska' => 'Anchorage'); # D'oh! Alaska isn't a Country
+    $tree->put('Alaska' => 'Anchorage'); # D'oh!
     $tree->delete('Alaska');
 
-    print scalar $tree->get('Ireland'); # 'Dublin'
+    print $tree->get('Ireland'); # 'Dublin'
 
-    print $tree->size; # 6
     print $tree->min->key; # 'Egypt' 
     print $tree->max->key; # 'Ireland' 
-
-    print $tree->nth(0)->key;  # 'Egypt' 
-    print $tree->nth(-1)->key; # 'Ireland' 
+    print $tree->size; # 6
 
     # print items, ordered by key
     my $it = $tree->iter;
 
     while(my $node = $it->next) {
-        printf "key = %s, value = %s\n", $node->key, $node->val;
+        sprintf "key = %s, value = %s\n", $node->key, $node->val;
     }
 
     # print items in reverse order
     $it = $tree->rev_iter;
 
     while(my $node = $it->next) {
-        printf "key = %s, value = %s\n", $node->key, $node->val;
+        sprintf "key = %s, value = %s\n", $node->key, $node->val;
     }
 
     # Hash interface
@@ -614,7 +507,7 @@ This is a Perl implementation of the Red/Black tree, a type of balanced binary s
 
 A tied hash interface is also provided to allow ordered hashes to be used.
 
-See the Wikipedia article at L<http://en.wikipedia.org/wiki/Red-black_tree> for further information about Red/Black trees.
+See the Wikipedia article at L<http://en.wikipedia.org/wiki/Red-black_tree> for more on Red/Black trees.
 
 
 =head1 INTERFACE
@@ -639,7 +532,7 @@ For example, to get a case insensitive ordering
     my $it = $tree->iter;
 
     while(my $node = $it->next) {
-        printf "key = %s, value = %s\n", $node->key, $node->val;
+        sprintf "key = %s, value = %s\n", $node->key, $node->val;
     }
 
 =head2 resort(CODEREF)
@@ -667,10 +560,6 @@ Returns the node with the minimal key.
 
 Returns the node with the maximal key.
 
-=head2 nth(INDEX)
-
-Returns the node at the given (zero based) index, or undef if there is no node at that index. Negative indexes can be used, with -1 indicating the last node, -2 the penultimate node and so on.
-
 =head2 lookup(KEY, [MODE])
 
 When called in scalar context, lookup(KEY) returns the value
@@ -692,7 +581,7 @@ Tree::RB
 
 =item LUEQUAL
 
-This is the default mode. Returns the node exactly matching the key, or C<undef> if not found. 
+Returns node exactly matching the key.
 
 =item LUGTEQ
 
@@ -736,66 +625,6 @@ This can be used to step through the tree in reverse order.
 
 get() is an alias for lookup().
 
-=head2 iter([KEY])
-
-Returns an iterator object that can be used to traverse the tree in order.
-
-The iterator object supports a 'next' method that returns the next node in the
-tree or undef if all of the nodes have been visited.
-
-See the synopsis for an example.
-
-If a key is supplied, the iterator returned will traverse the tree in order starting from
-the node with key greater than or equal to the specified key.
-
-    $it = $tree->iter('France');
-    my $node = $it->next;
-    print $node->key; # -> 'France'
-
-=head2 rev_iter([KEY])
-
-Returns an iterator object that can be used to traverse the tree in reverse order.
-
-If a key is supplied, the iterator returned will traverse the tree in order starting from
-the node with key less than or equal to the specified key.
-
-    $it = $tree->rev_iter('France');
-    my $node = $it->next;
-    print $node->key; # -> 'France'
-
-    $it = $tree->rev_iter('Finland');
-    my $node = $it->next;
-    print $node->key; # -> 'England'
-
-=head2 hseek(KEY, [{-reverse => 1|0}])
-
-For tied hashes, determines the next entry to be returned by each.
-
-    tie my %capital, 'Tree::RB';
-
-    $capital{'France'}  = 'Paris';
-    $capital{'England'} = 'London';
-    $capital{'Hungary'} = 'Budapest';
-    $capital{'Ireland'} = 'Dublin';
-    $capital{'Egypt'}   = 'Cairo';
-    $capital{'Germany'} = 'Berlin';
-    tied(%capital)->hseek('Germany');
-
-    ($key, $val) = each %capital;
-    print "$key, $val"; # -> Germany, Berlin 
-
-The direction of iteration can be reversed by passing a hashref with key '-reverse' and value 1
-to hseek after or instead of KEY, e.g. to iterate over the hash in reverse order:
-
-    tied(%capital)->hseek({-reverse => 1});
-    $key = each %capital;
-    print $key; # -> Ireland 
-
-The following calls are equivalent
-
-    tied(%capital)->hseek('Germany', {-reverse => 1});
-    tied(%capital)->hseek({-key => 'Germany', -reverse => 1});
-
 =head2 put(KEY, VALUE)
 
 Adds a new node to the tree. 
@@ -813,7 +642,7 @@ deleted from the tree and returned, otherwise C<undef> is returned.
 
 =head1 DEPENDENCIES
 
-L<enum>
+None.
 
 
 =head1 INCOMPATIBILITIES
@@ -823,8 +652,12 @@ None reported.
 
 =head1 BUGS AND LIMITATIONS
 
-Please report any bugs or feature requests via the GitHub web interface at 
-L<https://github.com/arunbear/perl5-red-black-tree/issues>.
+No bugs have been reported.
+
+Please report any bugs or feature requests to
+C<bug-tree-rb@rt.cpan.org>, or through the web interface at
+L<http://rt.cpan.org>.
+
 
 =head1 AUTHOR
 
@@ -833,10 +666,6 @@ Arun Prasad  C<< <arunbear@cpan.org> >>
 Some documentation has been borrowed from Benjamin Holzman's L<Tree::RedBlack>
 and Damian Ivereigh's libredblack (L<http://libredblack.sourceforge.net/>).
 
-=head1 ACKNOWLEDGEMENTS
-
-Thanks for bug reports go to Anton Petrusevich, Wes Thompson, Petre Mierlutiu, Tomer Vromen and Christopher Gurnee.
-
 =head1 LICENCE AND COPYRIGHT
 
 Copyright (c) 2007, Arun Prasad C<< <arunbear@cpan.org> >>. All rights reserved.
